package edu.uoc.pfc.statusdroid.activity;

import android.app.AlertDialog;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Toast;
import edu.uoc.pfc.statusdroid.R;
import edu.uoc.pfc.statusdroid.content.StatusDroidDB;
import edu.uoc.pfc.statusdroid.exception.StatusDroidException;
import edu.uoc.pfc.statusdroid.helper.CryptoHelper;
import edu.uoc.pfc.statusdroid.model.ServerConfigData;
import edu.uoc.pfc.statusdroid.utils.Constants;

public class ConfigActivity extends StatusDroidBaseActivity {

	// Tag de log
	private static final String LOG_TAG = "ConfigActivity";

	private int configId;
	private boolean edit;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.config);
	}

	@Override
	public void onResume() {
		super.onResume();
		Intent intent = getIntent();
		ServerConfigData configData = (ServerConfigData) intent
				.getParcelableExtra("serverConfig");
		if (configData != null) {
			this.edit = true;
			this.configId = configData.getId();

			// Omplim la vista
			((EditText) findViewById(R.id.config_server_name))
					.setText(configData.getName());
			((EditText) findViewById(R.id.config_server_url))
					.setText(configData.getUrl());

			if (configData.isAutconn()) {
				((CheckBox) findViewById(R.id.autoconn)).setChecked(true);
			}

			((EditText) findViewById(R.id.nom)).setText(configData.getUser());
			((EditText) findViewById(R.id.password)).setText(configData
					.getPassword());
		}

		// Afegim el botó d'esborrar
		if (edit) {
			findViewById(R.id.esborrar).setVisibility(View.VISIBLE);
		}
	}

	/**
	 * Mètode que conté les tasques a realitzar quan s'esborra una configuració.
	 * 
	 * @param v
	 *            Vista que ha produït l'event
	 */
	public void onDelete(View v) {
		// Esborrar
		Uri uri = ContentUris.withAppendedId(
				StatusDroidDB.ServerConfig.CONTENT_ID_URI, configId);
		getApplicationContext().getContentResolver().delete(uri, null, null);
		Toast.makeText(this, getResources().getString(R.string.info_delete_ok),
				Toast.LENGTH_SHORT).show();

		// Carregar Activitat
		Intent intent = new Intent(this, ConfigListActivity.class);
		startActivity(intent);

		finish();
	}

	/**
	 * Mètode que conté les tasques a realitzar quan es guarda una configuració.
	 * 
	 * @param v
	 *            Vista que ha produït l'event
	 */
	public void onSave(View v) {
		ContentValues values = new ContentValues();
		try {
			// NOM
			values.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_NAME,
					((EditText) findViewById(R.id.config_server_name))
							.getText().toString());

			// URI
			values.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_URI,
					((EditText) findViewById(R.id.config_server_url)).getText()
							.toString());

			// Autoconnectar
			if (((CheckBox) findViewById(R.id.autoconn)).isChecked()) {
				values.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_AUTOCONN,
						"true");
			} else {
				values.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_AUTOCONN,
						"false");
			}

			// User
			values.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_USER,
					((EditText) findViewById(R.id.nom)).getText().toString());

			// Password
			String passwordText = ((EditText) findViewById(R.id.password))
					.getText().toString();
			values.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_PASSWORD,
					encryptPassword(passwordText));

			// Validació
			validateInputData(values);

			// Actualització de dades
			if (!edit) {
				// Insert
				getApplicationContext().getContentResolver().insert(
						StatusDroidDB.ServerConfig.CONTENT_ID_URI, values);
				Toast.makeText(this,
						getResources().getString(R.string.info_save_ok),
						Toast.LENGTH_SHORT).show();
			} else {
				// Update
				Uri uri = ContentUris.withAppendedId(
						StatusDroidDB.ServerConfig.CONTENT_ID_URI, configId);
				getApplicationContext().getContentResolver().update(uri,
						values, null, null);
				Toast.makeText(this,
						getResources().getString(R.string.info_save_ok),
						Toast.LENGTH_SHORT).show();
			}
		} catch (StatusDroidException e) {
			showErrorAlert(e.getMessage());
			return;
		}
		Intent intent = new Intent(this, ConfigListActivity.class);
		startActivity(intent);

		finish();
	}

	/**
	 * Mètode d'utilitat que a partir d'un password en text clar el xifra per
	 * guardar-lo a la base de dades.
	 * 
	 * @param password
	 *            Password a xifrar
	 * @return Password xifrat
	 */
	private String encryptPassword(String password) throws StatusDroidException {
		String seed = getSharedPreferences(Constants.PREFS_NAME,
				Context.MODE_PRIVATE).getString(Constants.PREF_SEED, null);
		if (seed == null) {
			Log.e(LOG_TAG, getResources().getString(R.string.error_config_seed));
			throw new StatusDroidException(getResources().getString(
					R.string.error_config_seed));
		}
		String passwordEnc;
		try {
			passwordEnc = CryptoHelper.encrypt(seed, password);
		} catch (Exception e) {
			Log.e(LOG_TAG, e.getMessage(), e);
			throw new StatusDroidException(getResources().getString(
					R.string.error_config_save_pass), e);
		}
		return passwordEnc;
	}

	public void onCancel(View v) {
		showCancelAlert();
	}

	/**
	 * Mètode que s'encarrega de mostrar el missatge de cancel·lació de
	 * configuració
	 */
	public void showCancelAlert() {
		// prepare the alert box
		AlertDialog.Builder alertbox = new AlertDialog.Builder(this);

		// set the message to display
		alertbox.setMessage(getResources().getString(R.string.warn_cancel));

		// set a positive/yes button and create a listener
		alertbox.setPositiveButton(
				getResources().getString(R.string.button_acceptar),
				new DialogInterface.OnClickListener() {

					// do something when the button is clicked
					public void onClick(DialogInterface arg0, int arg1) {
						Intent myIntent = new Intent(getApplicationContext(),
								ConfigListActivity.class);
						startActivity(myIntent);
						finish();
					}
				});

		// set a negative/no button and create a listener
		alertbox.setNegativeButton(
				getResources().getString(R.string.button_cancel),
				new DialogInterface.OnClickListener() {

					// do something when the button is clicked
					public void onClick(DialogInterface arg0, int arg1) {

					}
				});

		// display box
		alertbox.show();
	}

	/**
	 * Mètode que valida les dades del formulari d'entrada de la configuració
	 * 
	 * @param values
	 *            Valors a validar
	 * @throws StatusDroidException
	 */
	private void validateInputData(ContentValues values)
			throws StatusDroidException {
		String name = values
				.getAsString(StatusDroidDB.ServerConfig.COLUMN_SERVER_NAME);
		String uri = values
				.getAsString(StatusDroidDB.ServerConfig.COLUMN_SERVER_URI);
		String user = values
				.getAsString(StatusDroidDB.ServerConfig.COLUMN_SERVER_USER);
		String password = values
				.getAsString(StatusDroidDB.ServerConfig.COLUMN_SERVER_PASSWORD);
		if (null == name || null == uri || null == user || null == password
				|| name.length() == 0 || uri.length() == 0) {
			throw new StatusDroidException(getResources().getString(
					R.string.error_config_data));
		}
	}
}
