package edu.uoc.pfc.statusdroid.activity;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;
import edu.uoc.pfc.statusdroid.R;
import edu.uoc.pfc.statusdroid.content.StatusDroidDB;
import edu.uoc.pfc.statusdroid.helper.ActionItem;
import edu.uoc.pfc.statusdroid.helper.QuickAction;
import edu.uoc.pfc.statusdroid.helper.ServerConfigArrayAdapter;
import edu.uoc.pfc.statusdroid.model.ServerConfigData;
import edu.uoc.pfc.statusdroid.model.User;
import edu.uoc.pfc.statusdroid.service.StatusNetAPIService;
import edu.uoc.pfc.statusdroid.utils.Constants;
import edu.uoc.pfc.statusdroid.utils.StatusDroidUtils;

public class ConfigListActivity extends StatusDroidBaseActivity {

	// Tag de log
	private static final String LOG_TAG = "ConfigListActivity";

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.config_list);
	}

	@Override
	protected void onResume() {
		super.onResume();
		if (mBound && null == statusNetAPI.getService().getCurrentServer()) {
			List<ServerConfigData> serverConfigList = loadConfigList();
			showConfigList(serverConfigList);
		}
	}

	@Override
	public void loadServiceAction() {
		List<ServerConfigData> serverConfigList = loadConfigList();
		showConfigList(serverConfigList);
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (StatusNetAPIService.mBound) {
			getApplicationContext().unbindService(mConnection);
			mBound = false;
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.statusdroid_menu, menu);
		return true;
	}

	/**
	 * Mètode que recupera la informació de les connexions d'StatusNet
	 * 
	 * @return Llistat de connexions
	 */
	private List<ServerConfigData> loadConfigList() {
		// 1 - Accedim a la base de dades per recuperar la configuració
		Cursor cursor = null;
		List<ServerConfigData> serverConfigList = new ArrayList<ServerConfigData>();
		try {
			cursor = getApplicationContext().getContentResolver().query(
					StatusDroidDB.ServerConfig.CONTENT_URI, null, null, null,
					null);
			cursor.moveToFirst();

			// 2 - Recuperar la configuració de les connexions
			ServerConfigData serverConfig = null;
			String seed = getSharedPreferences(Constants.PREFS_NAME,
					Context.MODE_PRIVATE).getString(Constants.PREF_SEED, null);

			while (!cursor.isAfterLast()) {
				if (seed != null) {
					serverConfig = StatusDroidUtils.recoverServerConfigData(
							cursor, seed);
				}
				serverConfigList.add(serverConfig);
				cursor.moveToNext();
			}

		} catch (Exception e) {
			Log.e(LOG_TAG,
					getResources().getString(R.string.error_recover_config), e);
			showErrorAlert(getResources().getString(
					R.string.error_recover_config));
		} finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return serverConfigList;
	}

	/**
	 * Mètode que mostra per pantalla el llistat de configuracions
	 * 
	 * @param serverConfigList
	 *            Llistat de configuracions a mostrar
	 */
	private void showConfigList(List<ServerConfigData> serverConfigList) {
		ListView configListView = (ListView) findViewById(R.id.config_list_view);

		ServerConfigArrayAdapter<ServerConfigData> adapter = new ServerConfigArrayAdapter<ServerConfigData>(
				this, R.layout.config_text_view, serverConfigList, statusNetAPI
						.getService().getCurrentServer());
		configListView
				.setOnItemClickListener(new AdapterView.OnItemClickListener() {

					public void onItemClick(AdapterView<?> parent, View view,
							int position, long id) {
						showConfigQuickAction(parent, view, position);
					}

				});
		configListView.setAdapter(adapter);
		configListView.invalidate();
	}

	/**
	 * Mètode que conté les tasques a realitzar quan es cancel·la la vista
	 * 
	 * @param v
	 *            Vista que ha produït l'event
	 */
	public void onCancel(View v) {
		Intent myIntent = new Intent(getApplicationContext(),
				MainActivity.class);
		startActivity(myIntent);
		finish();
	}

	/**
	 * Mètode que conté les tasques a realitzar quan es selecciona la creació
	 * d'una nova configuració.
	 * 
	 * @param v
	 *            Vista que ha produït l'event
	 */
	public void onNew(View v) {
		Intent myIntent = new Intent(getApplicationContext(),
				ConfigActivity.class);
		startActivity(myIntent);
	}

	/**
	 * Mètode d'utilitat que carrega el menú d'accions de configuració
	 * 
	 * @param parent
	 *            Vista de l'adaptador
	 * @param v
	 *            Vista de l'acció
	 * @param position
	 *            Posició de l'objecte
	 */
	public void showConfigQuickAction(final AdapterView<?> parent, View v,
			final int position) {

		QuickAction qa = new QuickAction(v);

		// Dades seleccionades
		final ServerConfigArrayAdapter<ServerConfigData> adapter = (ServerConfigArrayAdapter<ServerConfigData>) parent
				.getAdapter();
		final ServerConfigData configData = adapter.getItem(position);

		// Menú d'edició
		final ActionItem editMenu = new ActionItem();

		editMenu.setTitle(getResources().getString(R.string.action_menu_edit));
		editMenu.setIcon(getResources().getDrawable(R.drawable.edit_action));
		editMenu.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				Intent intent = new Intent(getApplicationContext(),
						ConfigActivity.class);

				intent.putExtra("serverConfig", configData);
				startActivity(intent);
			}
		});
		qa.addActionItem(editMenu);

		// Menú de connexió
		if (!statusNetAPI.getService().isConnected()
				|| !statusNetAPI.getService().getCurrentServer()
						.equals(configData)) {
			final ActionItem connectMenu = new ActionItem();

			connectMenu.setTitle(getResources().getString(
					R.string.action_menu_connect));
			connectMenu.setIcon(getResources().getDrawable(
					R.drawable.connect_icon));
			connectMenu.setOnClickListener(new OnClickListener() {

				public void onClick(View v) {
					// Connectem nova configuració
					statusNetAPI.getService().setCurrentServer(configData);
					statusNetAPI.getService().setConnected(true);

					// Carreguem el nou usuari
					loadUserData(statusNetAPI.getService().getCurrentServer()
							.getUser());
					// Actualitzem llistat
					adapter.setConnectedServerConfig(statusNetAPI.getService()
							.getCurrentServer());
					adapter.notifyDataSetChanged();

					Toast.makeText(
							getApplicationContext(),
							getResources().getString(
									R.string.info_connect_change),
							Toast.LENGTH_SHORT).show();
				}
			});
			qa.addActionItem(connectMenu);
		}
		qa.setAnimStyle(QuickAction.ANIM_GROW_FROM_CENTER);

		qa.show();
	}

	/**
	 * Mètode que carrega les dades de l'usuari actiu al servidor.
	 * 
	 * @param name
	 *            Nom de l'usuari a carregar
	 */
	private void loadUserData(String name) {
		// 1 - Accedim al servei per recuperar la informació
		try {
			User user = statusNetAPI.getUserInfoByName(name);
			statusNetAPI.getService().setUser(user);
		} catch (Exception e) {
			Log.e(LOG_TAG,
					getResources().getString(R.string.error_recover_config), e);
			showErrorAlert(getResources()
					.getString(R.string.error_recover_user));
		}
	}
}