package edu.uoc.pfc.statusdroid.activity;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Toast;
import edu.uoc.pfc.statusdroid.R;
import edu.uoc.pfc.statusdroid.exception.StatusDroidException;
import edu.uoc.pfc.statusdroid.helper.ActionItem;
import edu.uoc.pfc.statusdroid.helper.QuickAction;
import edu.uoc.pfc.statusdroid.model.Status;
import edu.uoc.pfc.statusdroid.model.User;
import edu.uoc.pfc.statusdroid.service.StatusNetAPIBinder;
import edu.uoc.pfc.statusdroid.service.StatusNetAPIService;

/**
 * Classe d¡activitat base que conté aquells mètodes comuns que poden ser
 * reutilitzats per més d'una activitat.
 * 
 * @author daniel
 * 
 */
public class StatusDroidBaseActivity extends Activity {

	StatusNetAPIBinder statusNetAPI;
	boolean mBound;

	/**
	 * Classe de Callback per a la connexió amb el servei d'StatusNet
	 * 
	 * @author daniel
	 * 
	 */
	protected class StatusDroidServiceConnection implements ServiceConnection {

		public void onServiceConnected(ComponentName className, IBinder service) {
			statusNetAPI = (StatusNetAPIBinder) service;
			mBound = true;
			loadServiceAction();
		}

		public void onServiceDisconnected(ComponentName arg0) {
			mBound = false;
		}
	}

	// Classe de Callback per a la connexió amb els serveis
	public ServiceConnection mConnection = new StatusDroidServiceConnection();

	/**
	 * Mètode d'utilitat per mostrar missatges d'error a l'usuari
	 * 
	 * @param message
	 *            Missatge a mostrar
	 */
	public void showErrorAlert(String message) {
		AlertDialog.Builder alertbox = new AlertDialog.Builder(this);
		alertbox.setMessage(message);

		// Acció a realitzar
		alertbox.setNeutralButton(getResources().getString(R.string.button_ok),
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface arg0, int arg1) {
						// No s'ha de fer res, tornar a la mateixa pantalla
					}
				});
		alertbox.show();
	}

	@Override
	protected void onStart() {
		super.onStart();
		// Enllaçar amb el servei d'StatusNet
		Intent intent = new Intent(this, StatusNetAPIService.class);
		getApplicationContext().bindService(intent, mConnection,
				Context.BIND_AUTO_CREATE);
	}

	/**
	 * Mètode per afegir-hi tasques addicionals a la connexió amb el servei
	 */
	public void loadServiceAction() {
		// Per defecte no fa res
	}

	@Override
	/**
	 * Mètode que tracta les accions del menú principal de l'aplicació
	 * @item menú que s'ha seleccionat
	 * @return resultat de l'acció
	 */
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent;
		switch (item.getItemId()) {
		case R.id.menu_home:
			intent = new Intent(this, MainActivity.class);
			startActivity(intent);
			break;
		case R.id.menu_config:
			intent = new Intent(this, ConfigListActivity.class);
			startActivity(intent);
			break;
		// case R.id.menu_help:
		// break;
		default:
			break;
		}
		return true;
	}

	/**
	 * Mètode d'utilitat que carrega el menú d'accions de missatges
	 * 
	 * @param parent
	 *            Vista de l'adaptador
	 * @param v
	 *            Vista de l'acció
	 * @param position
	 *            Posició de l'objecte
	 */
	public void showMessageQuickAction(final AdapterView<?> parent, View v,
			final int position) {

		QuickAction qa = new QuickAction(v);

		// Dades seleccionades
		ArrayAdapter<Status> adapter = (ArrayAdapter<Status>) parent
				.getAdapter();
		final Status status = adapter.getItem(position);

		// Menú de seguiment
		final ActionItem followMenu = new ActionItem();

		followMenu.setTitle(getResources().getString(
				R.string.action_menu_follow));
		followMenu
				.setIcon(getResources().getDrawable(R.drawable.follow_action));
		followMenu.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				User user;
				try {
					user = statusNetAPI.addFollower(String.valueOf(status
							.getUser().getId()));
				} catch (StatusDroidException e) {
					showErrorAlert(e.getMessage());
					return;
				}
				if (user != null) {
					Toast.makeText(getApplicationContext(),
							getResources().getString(R.string.info_following),
							Toast.LENGTH_SHORT).show();
				} else {
					Toast.makeText(
							getApplicationContext(),
							getResources().getString(
									R.string.info_already_following),
							Toast.LENGTH_SHORT).show();
				}
			}
		});
		qa.addActionItem(followMenu);

		// Menu de favorits
		final ActionItem favoriteMenu = new ActionItem();

		favoriteMenu.setTitle(getResources().getString(
				R.string.action_menu_favorite));
		favoriteMenu.setIcon(getResources().getDrawable(
				R.drawable.favorite_action));
		favoriteMenu.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				boolean resultat;
				try {
					resultat = statusNetAPI.addFavorite(String.valueOf(status
							.getId()));
				} catch (StatusDroidException e) {
					showErrorAlert(e.getMessage());
					return;
				}
				if (resultat) {
					Toast.makeText(getApplicationContext(),
							getResources().getString(R.string.info_favorites),
							Toast.LENGTH_SHORT).show();
				} else {
					Toast.makeText(
							getApplicationContext(),
							getResources().getString(
									R.string.info_already_favorites),
							Toast.LENGTH_SHORT).show();
				}
			}
		});
		qa.addActionItem(favoriteMenu);

		// Menú d'info usuari
		final ActionItem userMenu = new ActionItem();

		userMenu.setTitle(getResources().getString(R.string.action_menu_user));
		userMenu.setIcon(getResources().getDrawable(R.drawable.info_action));
		userMenu.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				Intent intent = new Intent(getApplicationContext(),
						UserActivity.class);
				intent.putExtra("userId", status.getUser().getId());
				startActivity(intent);
			}
		});
		qa.addActionItem(userMenu);

		qa.setAnimStyle(QuickAction.ANIM_GROW_FROM_CENTER);

		qa.show();
	}
}
