package edu.uoc.pfc.statusdroid.content;

import java.util.HashMap;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.text.TextUtils;

public class StatusDroidContentProvider extends ContentProvider {

	private static final String DATABASE_NAME = "statusdroid.db";

	private static final int DATABASE_VERSION = 3;

	/**
	 * Projecció de les columnes de la base de dades per recuperarles a paritr
	 * dle nom
	 */
	private static HashMap<String, String> serverConfigDBMap;

	private static final UriMatcher uriMatcher;

	// URI associada al llistat de configuracions
	private static final int SERVER_CONFIG = 1;

	// URI associada a una configuració en concret
	private static final int SERVER_CONFIG_ID = 2;

	private StatusDroidDBHelper statusDroidDBHelper;

	static {

		// Configuració del matcher per les peticions al proveidor
		uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);

		// Ruta per a totes les configuracions
		uriMatcher.addURI(StatusDroidDB.AUTHORITY, "serverConfig",
				SERVER_CONFIG);

		// Ruta per recuperar una configuració concreta a partir del seu id
		uriMatcher.addURI(StatusDroidDB.AUTHORITY, "serverConfig/#",
				SERVER_CONFIG_ID);

		// Configuració del mapeig de les columnes
		serverConfigDBMap = new HashMap<String, String>();

		// Columna ID
		serverConfigDBMap.put(StatusDroidDB.ServerConfig._ID,
				StatusDroidDB.ServerConfig._ID);

		// Columna Nom del servidor
		serverConfigDBMap.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_NAME,
				StatusDroidDB.ServerConfig.COLUMN_SERVER_NAME);
		// Columna URI del servidor
		serverConfigDBMap.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_URI,
				StatusDroidDB.ServerConfig.COLUMN_SERVER_URI);
		// Columna Autoconnectar
		serverConfigDBMap.put(
				StatusDroidDB.ServerConfig.COLUMN_SERVER_AUTOCONN,
				StatusDroidDB.ServerConfig.COLUMN_SERVER_AUTOCONN);
		// Columna Nom Usuari
		serverConfigDBMap.put(StatusDroidDB.ServerConfig.COLUMN_SERVER_USER,
				StatusDroidDB.ServerConfig.COLUMN_SERVER_USER);
		// Columna Password
		serverConfigDBMap.put(
				StatusDroidDB.ServerConfig.COLUMN_SERVER_PASSWORD,
				StatusDroidDB.ServerConfig.COLUMN_SERVER_PASSWORD);

	}

	@Override
	public int delete(Uri uri, String selection, String[] selectionArgs) {
		// 1 - Obrir connexió a la base de dades per a escriptura
		SQLiteDatabase db = statusDroidDBHelper.getWritableDatabase();
		int count;

		// 2 - Selecció de l'actuació a partir de l'URI
		switch (uriMatcher.match(uri)) {
		case SERVER_CONFIG:
			// Esborra totes les entrades que compleixin la selecció d'entrada
			count = db.delete(StatusDroidDB.ServerConfig.TABLE_NAME, selection,
					selectionArgs);
			break;

		case SERVER_CONFIG_ID:
			// Esborra només les dades associades a un ID concret
			String selectionById = StatusDroidDB.ServerConfig._ID
					+ " = "
					+ uri.getPathSegments()
							.get(StatusDroidDB.ServerConfig.SERVERCONFIG_ID_POSITION);

			// Construcció de la clausula de selecció
			if (selection != null) {
				selectionById = selectionById + " AND " + selection;
			}

			// Execució del delete
			count = db.delete(StatusDroidDB.ServerConfig.TABLE_NAME,
					selectionById, selectionArgs);
			break;

		default:
			// En altre cas es produeix una excepció d'uri desconeguda
			throw new IllegalArgumentException("URI desconeguda: " + uri);
		}

		// 3 - Notificació del canvi
		getContext().getContentResolver().notifyChange(uri, null);

		return count;
	}

	@Override
	public String getType(Uri uri) {
		switch (uriMatcher.match(uri)) {
		case SERVER_CONFIG:
			// Llistat de configuracions
			return StatusDroidDB.ServerConfig.CONTENT_TYPE;
		case SERVER_CONFIG_ID:
			// Un Item de configuració concret
			return StatusDroidDB.ServerConfig.CONTENT_ITEM_TYPE;
		default:
			// En altre cas es produeix una excepció d'uri desconeguda
			throw new IllegalArgumentException("URI desconeguda: " + uri);
		}

	}

	@Override
	public Uri insert(Uri uri, ContentValues values) {
		// 1 - Valida que la URI de la petició compleix amb la URI del proveïdor
		if (uriMatcher.match(uri) != SERVER_CONFIG) {
			// En altre cas es produeix una excepció d'uri desconeguda
			throw new IllegalArgumentException("URI desconeguda: " + uri);
		}

		// 2 - Recuperar valors a insertar
		ContentValues insertValues;

		// If the incoming values map is not null, uses it for the new values.
		if (values != null) {
			insertValues = new ContentValues(values);

		} else {
			// Otherwise, create a new value map
			insertValues = new ContentValues();
		}

		// // Gets the current system time in milliseconds
		// Long now = Long.valueOf(System.currentTimeMillis());

		// 3 - Obrir connexió a la base de dades per a escriptura
		SQLiteDatabase db = statusDroidDBHelper.getWritableDatabase();

		// 4 - Execució de l'insert
		long configId = db.insert(StatusDroidDB.ServerConfig.TABLE_NAME, null,
				insertValues);

		// 5 - Tractament resultat
		if (configId > 0) {
			// Creació nova URI
			Uri noteUri = ContentUris.withAppendedId(
					StatusDroidDB.ServerConfig.CONTENT_ID_URI, configId);

			// Notificació del canvi de dades
			getContext().getContentResolver().notifyChange(noteUri, null);
			return noteUri;
		}

		// Error a la inserció
		throw new SQLException("Error insertat fila: " + uri);

	}

	@Override
	public boolean onCreate() {
		statusDroidDBHelper = new StatusDroidDBHelper(getContext());
		return true;
	}

	@Override
	public Cursor query(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder) {

		// 1 - Creació de l'objecte Query
		SQLiteQueryBuilder qb = new SQLiteQueryBuilder();
		qb.setTables(StatusDroidDB.ServerConfig.TABLE_NAME);

		// 2 - Selecció del tipus de consulta a partir de la URI
		switch (uriMatcher.match(uri)) {
		case SERVER_CONFIG:
			// Recuperar totes les configuracions sense filtre
			qb.setProjectionMap(serverConfigDBMap);
			break;

		case SERVER_CONFIG_ID:
			// Recuperar les configuracions filtrant per ID
			qb.setProjectionMap(serverConfigDBMap);
			qb.appendWhere(StatusDroidDB.ServerConfig._ID
					+ '='
					+ uri.getPathSegments()
							.get(StatusDroidDB.ServerConfig.SERVERCONFIG_ID_POSITION));
			break;

		default:
			// En altre cas es produeix una excepció d'uri desconeguda
			throw new IllegalArgumentException("URI desconeguda: " + uri);
		}

		// 3 - Ordenació
		String orderBy;
		// If no sort order is specified, uses the default
		if (TextUtils.isEmpty(sortOrder)) {
			orderBy = StatusDroidDB.ServerConfig.DEFAULT_SORT_ORDER;
		} else {
			// otherwise, uses the incoming sort order
			orderBy = sortOrder;
		}

		// 4 - Obrir connexió a la base de dades per a consulta (Read Only)
		SQLiteDatabase db = statusDroidDBHelper.getReadableDatabase();

		// 5 - Execució de la query
		Cursor cursorResult = qb.query(db, projection, selection,
				selectionArgs, null, null, orderBy);

		// 6 - Configurar el cursor per detectar canvis
		cursorResult.setNotificationUri(getContext().getContentResolver(), uri);
		return cursorResult;

	}

	@Override
	public int update(Uri uri, ContentValues values, String selection,
			String[] selectionArgs) {

		// 1 - Obrir connexió a la base de dades per a escriptura
		SQLiteDatabase db = statusDroidDBHelper.getWritableDatabase();
		int count;

		// 2 - Selecció de l'actuació a partir de l'URI
		switch (uriMatcher.match(uri)) {
		case SERVER_CONFIG:
			// Actualitza totes les entrades que compleixin la selecció
			// d'entrada
			count = db.update(StatusDroidDB.ServerConfig.TABLE_NAME, values,
					selection, selectionArgs);
			break;

		case SERVER_CONFIG_ID:
			// Actualitza només les dades associades a un ID concret
			String selectionById = StatusDroidDB.ServerConfig._ID
					+ " = "
					+ uri.getPathSegments()
							.get(StatusDroidDB.ServerConfig.SERVERCONFIG_ID_POSITION);

			// Construcció de la clausula de selecció
			if (selection != null) {
				selectionById = selectionById + " AND " + selection;
			}

			// Execució del delete
			count = db.update(StatusDroidDB.ServerConfig.TABLE_NAME, values,
					selectionById, selectionArgs);
			break;

		default:
			// En altre cas es produeix una excepció d'uri desconeguda
			throw new IllegalArgumentException("URI desconeguda: " + uri);
		}

		// 3 - Notificació del canvi
		getContext().getContentResolver().notifyChange(uri, null);

		return count;
	}

	static class StatusDroidDBHelper extends SQLiteOpenHelper {

		StatusDroidDBHelper(Context context) {

			// calls the super constructor, requesting the default cursor
			// factory.
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		/**
		 * 
		 * Creates the underlying database with table name and column names
		 * taken from the NotePad class.
		 */
		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL("CREATE TABLE " + StatusDroidDB.ServerConfig.TABLE_NAME
					+ " (" + StatusDroidDB.ServerConfig._ID
					+ " INTEGER PRIMARY KEY AUTOINCREMENT,"
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_NAME + ' '
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_NAME_TYPE + ','
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_URI + ' '
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_URI_TYPE + ','
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_AUTOCONN + ' '
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_AUTOCONN_TYPE
					+ ',' + StatusDroidDB.ServerConfig.COLUMN_SERVER_USER + ' '
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_USER_TYPE + ','
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_PASSWORD + ' '
					+ StatusDroidDB.ServerConfig.COLUMN_SERVER_PASSWORD_TYPE
					+ ");");
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {

			// Kills the table and existing data
			db.execSQL("DROP TABLE IF EXISTS "
					+ StatusDroidDB.ServerConfig.TABLE_NAME);

			// Recreates the database with a new version
			onCreate(db);
		}

	}
}
