package edu.uoc.pfc.statusdroid.service;

import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.codehaus.jackson.JsonFactory;
import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.map.DeserializationConfig;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;

import android.app.Service;
import android.os.Binder;
import edu.uoc.pfc.statusdroid.R;
import edu.uoc.pfc.statusdroid.exception.StatusDroidException;
import edu.uoc.pfc.statusdroid.helper.Base64;
import edu.uoc.pfc.statusdroid.model.Status;
import edu.uoc.pfc.statusdroid.model.User;
import edu.uoc.pfc.statusdroid.utils.StatusDroidUtils;

public class StatusNetAPIBinder extends Binder {

	// Tag de Log
	private static final String LOG_TAG = "StatusNetAPIBinder";

	StatusNetAPIService service = null;

	public StatusNetAPIService getService() {
		return service;
	}

	public void setService(Service service) {
		this.service = (StatusNetAPIService) service;
	}

	public StatusNetAPIBinder(Service service) {
		this.service = (StatusNetAPIService) service;
	}

	/**
	 * Mètode que recupera la versió del servidor StatusNet
	 * 
	 * @return Versió del servidor
	 * @throws StatusDroidException
	 */
	public String getVersion() throws StatusDroidException {
		HttpClient client = new DefaultHttpClient();
		HttpGet httpGet = new HttpGet(service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_version));
		HttpResponse response = null;
		String versio = null;
		try {
			response = client.execute(httpGet);
			JsonFactory jsonFactory = new JsonFactory();
			JsonParser jsonParser = jsonFactory.createJsonParser(response
					.getEntity().getContent());
			jsonParser.nextToken();
			versio = jsonParser.getText();
		} catch (Exception e) {
			StatusDroidUtils.generateError(LOG_TAG,
					service.getString(R.string.error_statusnet), e);
		}

		return versio;
	}

	/**
	 * Mètode de l'API que recupera la línia de temps pública de missatges
	 * d'StatusNet
	 * 
	 * @return Llistat dels missatges recuperats
	 * @throws StatusDroidException
	 */
	public List<Status> getPublicTimeLine() throws StatusDroidException {
		String url = service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_public_timeline);
		return getTimeLine(url);
	}

	/**
	 * Mètode de l'API que recupera la línia de temps personal de missatges
	 * d'StatusNet
	 * 
	 * @return Llistat dels missatges recuperats
	 * @throws StatusDroidException
	 */
	public List<Status> getHomeTimeLine() throws StatusDroidException {
		String url = service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_home_timeline);
		return getTimeLine(url);
	}

	public List<Status> getFavorites() throws StatusDroidException {
		String url = service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_favorites);
		return getTimeLine(url);
	}

	/**
	 * Mètode que recupera una línia temporal a partir de la URL
	 * 
	 * @param url
	 *            URL de la línia temporal a recuperar
	 * @return Llistat d'estats recuperat
	 * @throws StatusDroidException
	 */
	private List<Status> getTimeLine(String url) throws StatusDroidException {
		HttpClient client = new DefaultHttpClient();
		HttpGet httpPost = new HttpGet(url);

		// Autorització
		httpPost.addHeader(
				"Authorization",
				"Basic "
						+ Base64.encodeBytes((service.getCurrentServer()
								.getUser() + ":" + service.getCurrentServer()
								.getPassword()).getBytes()));

		HttpResponse response = null;
		List<Status> statuses = null;
		try {
			response = client.execute(httpPost);
			if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
				StatusDroidUtils.generateError(LOG_TAG,
						service.getString(R.string.error_api_http)
								+ response.getStatusLine().getReasonPhrase(),
						null);
			}
			JsonFactory jsonFactory = new JsonFactory();
			JsonParser jsonParser = jsonFactory.createJsonParser(response
					.getEntity().getContent());
			ObjectMapper mapper = new ObjectMapper();
			mapper.configure(
					DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES,
					false);
			statuses = mapper.readValue(jsonParser,
					new TypeReference<List<Status>>() {
					});
		} catch (StatusDroidException e) {
			throw e;
		} catch (Exception e) {
			StatusDroidUtils.generateError(LOG_TAG,
					service.getString(R.string.error_statusnet), e);
		}
		return statuses;
	}

	/**
	 * Mètode de l'API per enviar actualitzacions d'estat amb nous missatges al
	 * servidor.
	 * 
	 * @param message
	 *            Missatge a enviar
	 * 
	 * @throws StatusDroidException
	 */
	public void sendUpdate(String message) throws StatusDroidException {
		sendUpdate(message, null);
	}

	/**
	 * Mètode de l'API per enviar actualitzacions d'estat amb nous missatges al
	 * servidor.
	 * 
	 * @param message
	 *            Missatge a enviar
	 * @param replyToId
	 *            Id a la qual s'està responent
	 * @throws StatusDroidException
	 */
	public void sendUpdate(String message, String replyToId)
			throws StatusDroidException {
		String url = service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_update);

		HttpClient client = new DefaultHttpClient();
		HttpPost httpPost = new HttpPost(url);

		// Autorització
		httpPost.addHeader(
				"Authorization",
				"Basic "
						+ Base64.encodeBytes((service.getCurrentServer()
								.getUser() + ":" + service.getCurrentServer()
								.getPassword()).getBytes()));

		HttpResponse response = null;
		try {
			// Paràmetres
			List<NameValuePair> params = new ArrayList<NameValuePair>();
			params.add(new BasicNameValuePair("status", message));
			if (null != replyToId) {
				params.add(new BasicNameValuePair("in_reply_to_status_id",
						replyToId));
			}
			httpPost.setEntity(new UrlEncodedFormEntity(params, HTTP.UTF_8));
			response = client.execute(httpPost);

			if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
				StatusDroidUtils.generateError(LOG_TAG,
						service.getString(R.string.error_api_http)
								+ response.getStatusLine().getReasonPhrase(),
						null);
			}

		} catch (Exception e) {
			StatusDroidUtils.generateError(LOG_TAG,
					service.getString(R.string.error_statusnet), e);
		}
	}

	/**
	 * Mètode que recupera les dades d'un usuari a partir del seu id
	 * 
	 * @param id
	 *            id de l'usuari
	 * @return Usuari recuperat
	 * @throws StatusDroidException
	 */
	public User getUserInfoById(String id) throws StatusDroidException {
		return getUserInfo(id, null);
	}

	/**
	 * Mètode que recupera les dades d'un usuari a partir del seu nom
	 * 
	 * @param name
	 *            nom de l'usuari
	 * @return Usuari recuperat
	 * @throws StatusDroidException
	 */
	public User getUserInfoByName(String name) throws StatusDroidException {
		return getUserInfo(null, name);
	}

	/**
	 * Mètode que recupera les dades d'un usuari a partir del seu id o nom per
	 * pantalla
	 * 
	 * @param id
	 *            id de l'usuari
	 * @param name
	 *            nom de l'usuari
	 * @return Usuari recuperat
	 * @throws StatusDroidException
	 */
	public User getUserInfo(String id, String name) throws StatusDroidException {
		HttpClient client = new DefaultHttpClient();

		String url = service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_users_show);
		HttpPost httpPost = new HttpPost(url);

		// Autorització
		httpPost.addHeader(
				"Authorization",
				"Basic "
						+ Base64.encodeBytes((service.getCurrentServer()
								.getUser() + ":" + service.getCurrentServer()
								.getPassword()).getBytes()));

		HttpResponse response = null;
		User user = null;
		try {
			// Paràmetres
			List<NameValuePair> params = new ArrayList<NameValuePair>();
			if (null != id) {
				params.add(new BasicNameValuePair("user_id", id));
			}
			if (null != name) {
				params.add(new BasicNameValuePair("screen_name", name));
			}
			httpPost.setEntity(new UrlEncodedFormEntity(params, HTTP.UTF_8));
			response = client.execute(httpPost);
			if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
				StatusDroidUtils.generateError(LOG_TAG,
						service.getString(R.string.error_api_http)
								+ response.getStatusLine().getReasonPhrase(),
						null);
			}
			JsonFactory jsonFactory = new JsonFactory();
			JsonParser jsonParser = jsonFactory.createJsonParser(response
					.getEntity().getContent());
			ObjectMapper mapper = new ObjectMapper();
			mapper.configure(
					DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES,
					false);
			user = mapper.readValue(jsonParser, User.class);
		} catch (StatusDroidException e) {
			throw e;
		} catch (Exception e) {
			StatusDroidUtils.generateError(LOG_TAG,
					service.getString(R.string.error_statusnet), e);
		}
		return user;
	}

	/**
	 * Mètode que afegeix l'usuari indicat a la llista de followers
	 * 
	 * @param id
	 *            id de l'usuari
	 * @return Usuari recuperat
	 * @throws StatusDroidException
	 */
	public User addFollower(String id) throws StatusDroidException {
		HttpClient client = new DefaultHttpClient();

		String url = service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_follow);
		HttpPost httpPost = new HttpPost(url);

		// Autorització
		httpPost.addHeader(
				"Authorization",
				"Basic "
						+ Base64.encodeBytes((service.getCurrentServer()
								.getUser() + ":" + service.getCurrentServer()
								.getPassword()).getBytes()));

		HttpResponse response = null;
		User user = null;
		try {
			// Paràmetres
			List<NameValuePair> params = new ArrayList<NameValuePair>();
			params.add(new BasicNameValuePair("user_id", id));
			params.add(new BasicNameValuePair("follow", "true"));
			httpPost.setEntity(new UrlEncodedFormEntity(params, HTTP.UTF_8));
			response = client.execute(httpPost);
			if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
				if (response.getStatusLine().getStatusCode() == HttpStatus.SC_FORBIDDEN) {
					// Ja s'està seguint l'usuari
					return null;
				}
				StatusDroidUtils.generateError(LOG_TAG,
						service.getString(R.string.error_api_http)
								+ response.getStatusLine().getReasonPhrase(),
						null);
			}
			JsonFactory jsonFactory = new JsonFactory();
			JsonParser jsonParser = jsonFactory.createJsonParser(response
					.getEntity().getContent());
			ObjectMapper mapper = new ObjectMapper();
			mapper.configure(
					DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES,
					false);
			user = mapper.readValue(jsonParser, User.class);
		} catch (StatusDroidException e) {
			throw e;
		} catch (Exception e) {
			StatusDroidUtils.generateError(LOG_TAG,
					service.getString(R.string.error_statusnet), e);
		}
		return user;
	}

	/**
	 * Mètode que afegeix l'status indicat a la llista de favorits
	 * 
	 * @param id
	 *            id de l'status a afegir
	 * @return resultat de l'acció
	 * @throws StatusDroidException
	 */
	public boolean addFavorite(String id) throws StatusDroidException {
		HttpClient client = new DefaultHttpClient();

		String url = service.getCurrentServer().getUrl()
				+ service.getString(R.string.api_create_favorite) + id
				+ ".json";
		HttpPost httpPost = new HttpPost(url);

		// Autorització
		httpPost.addHeader(
				"Authorization",
				"Basic "
						+ Base64.encodeBytes((service.getCurrentServer()
								.getUser() + ":" + service.getCurrentServer()
								.getPassword()).getBytes()));

		HttpResponse response = null;
		User user = null;
		try {

			response = client.execute(httpPost);
			if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
				if (response.getStatusLine().getStatusCode() == HttpStatus.SC_FORBIDDEN) {
					// Ja s'està seguint l'usuari
					return false;
				}
				StatusDroidUtils.generateError(LOG_TAG,
						service.getString(R.string.error_api_http)
								+ response.getStatusLine().getReasonPhrase(),
						null);
			}
			JsonFactory jsonFactory = new JsonFactory();
			JsonParser jsonParser = jsonFactory.createJsonParser(response
					.getEntity().getContent());
			ObjectMapper mapper = new ObjectMapper();
			mapper.configure(
					DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES,
					false);
			user = mapper.readValue(jsonParser, User.class);
		} catch (StatusDroidException e) {
			throw e;
		} catch (Exception e) {
			StatusDroidUtils.generateError(LOG_TAG,
					service.getString(R.string.error_statusnet), e);
		}
		return true;
	}
}
